@@ -25,11 +25,16 @@
 
 #include "services/auth/Types.hpp"
 
+namespace lms::auth
+{
+    class IPasswordService;
+}
+
 namespace lms::ui
 {
     using PasswordValidationContextGetFunc = std::function<auth::PasswordValidationContext()>;
-    std::unique_ptr<Wt::WValidator> createPasswordStrengthValidator(PasswordValidationContextGetFunc passwordValidationContextGetFunc);
+    std::unique_ptr<Wt::WValidator> createPasswordStrengthValidator(const auth::IPasswordService& passwordService, PasswordValidationContextGetFunc passwordValidationContextGetFunc);
 
     // Check current user password
-    std::unique_ptr<Wt::WValidator> createPasswordCheckValidator();
+    std::unique_ptr<Wt::WValidator> createPasswordCheckValidator(auth::IPasswordService& passwordService);
 } // namespace lms::ui
