@@ -33,6 +33,7 @@
 #include "database/Session.hpp"
 #include "database/Track.hpp"
 #include "database/TrackArtistLink.hpp"
+#include "database/TrackLyrics.hpp"
 #include "database/User.hpp"
 #include "services/feedback/IFeedbackService.hpp"
 #include "services/scrobbling/IScrobblingService.hpp"
@@ -43,7 +44,7 @@
 #include "Utils.hpp"
 #include "common/Template.hpp"
 #include "explore/PlayQueueController.hpp"
-#include "resource/CoverResource.hpp"
+#include "resource/ArtworkResource.hpp"
 #include "resource/DownloadResource.hpp"
 
 namespace lms::ui::TrackListHelpers
@@ -155,6 +156,52 @@ namespace lms::ui::TrackListHelpers
         LmsApp->getModalManager().show(std::move(trackInfo));
     }
 
+    void showTrackLyricsModal(db::TrackId trackId)
+    {
+        auto transaction{ LmsApp->getDbSession().createReadTransaction() };
+
+        auto trackLyrics{ std::make_unique<Template>(Wt::WString::tr("Lms.Explore.Tracks.template.track-lyrics")) };
+        Wt::WWidget* trackLyricsPtr{ trackLyrics.get() };
+        trackLyrics->addFunction("tr", &Wt::WTemplate::Functions::tr);
+
+        Wt::WContainerWidget* lyricsContainer{ trackLyrics->bindNew<Wt::WContainerWidget>("lyrics") };
+
+        // limitation: display only first lyrics for now
+        db::TrackLyrics::FindParameters params;
+        params.setTrack(trackId);
+        params.setSortMethod(db::TrackLyricsSortMethod::ExternalFirst);
+        params.setRange(db::Range{ 0, 1 });
+
+        db::TrackLyrics::find(LmsApp->getDbSession(), params, [&](const db::TrackLyrics::pointer& lyrics) {
+            std::string text;
+            auto addLine = [&](std::string_view line) {
+                if (!text.empty())
+                    text += '\n';
+                text += line;
+            };
+
+            if (lyrics->isSynchronized())
+            {
+                for (const auto& [timestamp, line] : lyrics->getSynchronizedLines())
+                    addLine(line);
+            }
+            else
+            {
+                for (const auto& line : lyrics->getUnsynchronizedLines())
+                    addLine(line);
+            }
+
+            lyricsContainer->addNew<Wt::WText>(Wt::WString::fromUTF8(text), Wt::TextFormat::Plain);
+        });
+
+        Wt::WPushButton* okBtn{ trackLyrics->bindNew<Wt::WPushButton>("ok-btn", Wt::WString::tr("Lms.ok")) };
+        okBtn->clicked().connect([=] {
+            LmsApp->getModalManager().dispose(trackLyricsPtr);
+        });
+
+        LmsApp->getModalManager().show(std::move(trackLyrics));
+    }
+
     std::unique_ptr<Wt::WWidget> createEntry(const db::ObjectPtr<db::Track>& track, PlayQueueController& playQueueController, Filters& filters)
     {
         auto entry{ std::make_unique<Template>(Wt::WString::tr("Lms.Explore.Tracks.template.entry")) };
@@ -173,20 +220,20 @@ namespace lms::ui::TrackListHelpers
             entry->bindWidget("artists-md", utils::createArtistDisplayNameWithAnchors(track->getArtistDisplayName(), artists));
         }
 
+        auto image{ utils::createTrackImage(trackId, ArtworkResource::Size::Small) };
+        image->addStyleClass("Lms-cover-track");
         if (track->getRelease())
         {
             entry->setCondition("if-has-release", true);
             entry->bindWidget("release", utils::createReleaseAnchor(track->getRelease()));
+
             Wt::WAnchor* anchor{ entry->bindWidget("cover", utils::createReleaseAnchor(release, false)) };
-            auto cover{ utils::createCover(release->getId(), CoverResource::Size::Small) };
-            cover->addStyleClass("Lms-cover-track Lms-cover-anchor"); // HACK
-            anchor->setImage(std::move((cover)));
+            image->addStyleClass("Lms-cover-anchor"); // HACK
+            anchor->setImage(std::move((image)));
         }
         else
         {
-            auto cover{ utils::createCover(trackId, CoverResource::Size::Small) };
-            cover->addStyleClass("Lms-cover-track"); // HACK
-            entry->bindWidget<Wt::WImage>("cover", std::move(cover));
+            entry->bindWidget<Wt::WImage>("cover", std::move(image));
         }
 
         entry->bindString("duration", utils::durationToString(track->getDuration()), Wt::TextFormat::Plain);
@@ -240,6 +287,14 @@ namespace lms::ui::TrackListHelpers
             ->clicked()
             .connect([trackId, &filters] { showTrackInfoModal(trackId, filters); });
 
+        if (track->hasLyrics())
+        {
+            entry->setCondition("if-has-lyrics", true);
+            entry->bindNew<Wt::WPushButton>("track-lyrics", Wt::WString::tr("Lms.Explore.track-lyrics"))
+                ->clicked()
+                .connect([trackId] { showTrackLyricsModal(trackId); });
+        }
+
         LmsApp->getMediaPlayer().trackLoaded.connect(entryPtr, [=](db::TrackId loadedTrackId) {
             entryPtr->toggleStyleClass("Lms-entry-playing", loadedTrackId == trackId);
         });
